package de.bsvrz.buv.plugin.darstellung.commands;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;

import com.bitctrl.lib.eclipse.emf.gef.model.Sized;

import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;

/**
 * Legt ein neues Darstellungsobjekts in einer Ebene oder Darstellungsspalte an.
 * 
 * <p>
 * <em>Hinweis:</em> Die Position und ggf. Gre des neuen Darstellungsobjekts
 * muss mit {@link #setLocation(Point)} angegeben werden, wenn im Modell keine
 * Position angegeben ist.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 * @see #setLocation(Point)
 */
public class CreateDoModelCommand extends Command {

	private final DoModel model;
	private final Ebene ebene;
	private final DarstellungsSpalte darstellungsSpalte;
	private Point location;
	private Dimension size;

	public CreateDoModelCommand(final DoModel model, final Ebene ebene) {
		this(model, ebene, null);
	}

	public CreateDoModelCommand(final DoModel model, final Ebene ebene,
			final DarstellungsSpalte darstellungsSpalte) {
		Assert.isNotNull(model, "model");
		Assert.isNotNull(ebene, "ebene");
		assertDarstellungsSpalteNeeded(ebene, darstellungsSpalte);

		this.model = model;
		this.ebene = ebene;
		this.darstellungsSpalte = darstellungsSpalte;

		setLabel("Darstellungsonjekt anlegen");
	}

	private void assertDarstellungsSpalteNeeded(final Ebene e,
			final DarstellungsSpalte s) {
		if (e.getDarstellung() instanceof StilisierteDarstellung) {
			final StilisierteDarstellung darstellung = (StilisierteDarstellung) e
					.getDarstellung();
			if (!darstellung.getSpalten().isEmpty()) {
				Assert.isNotNull(s, "darstellungsSpalte");
			}
		}
	}

	public Point getLocation() {
		return location;
	}

	public void setLocation(final Point location) {
		this.location = location;
	}

	public Dimension getSize() {
		return size;
	}

	public void setSize(final Dimension size) {
		this.size = size;
	}

	@Override
	public boolean canExecute() {
		return location != null || model.getLocation() != null;
	}

	@Override
	public void execute() {
		Assert.isTrue(canExecute(), "command is not executable");

		applyConstraintsIfNecessary();

		redo();
	}

	private void applyConstraintsIfNecessary() {
		if (getLocation() != null) {
			model.setLocation(getLocation());
		} else {
			Assert.isNotNull(model.getLocation(),
					"entweder constraint oder location muss gesetzt sein");
		}
		if (model instanceof Sized && getSize() != null) {
			((Sized) model).setSize(getSize());
		}
	}

	@Override
	public void redo() {
		ebene.getDoObjekte().add(model);
		if (darstellungsSpalte != null) {
			darstellungsSpalte.getDoObjekte().add(model);
		}
	}

	@Override
	public void undo() {
		if (darstellungsSpalte != null) {
			darstellungsSpalte.getDoObjekte().remove(model);
		}
		ebene.getDoObjekte().remove(model);
	}

}
