package de.bsvrz.buv.plugin.dobj.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.plugin.doeditor.model.DoeditorFactory;
import de.bsvrz.buv.plugin.doeditor.model.EditorDoTypReferenz;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmdarstellungsobjekteglobal.objekte.DarstellungsObjektTyp;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hilfsklasse fr den Zugriff auf die im System vorhandenen
 * Darstellungsobjekttypen.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class DoTypenUtil {

	/**
	 * Gibt alle Darstellungsobjekttypen zurck, die zur Verfgung stehen.
	 * 
	 * @return alle bekannten Darstellungsobjekttypen.
	 */
	public static Collection<DoTyp> getDoTypen() {
		final Collection<DoTyp> typen = new ArrayList<DoTyp>();

		// Programmierte DO-Typen
		typen.addAll(DarstellungExtensionPoint.getDoTypen());

		// Editor-DO-Typen
		final ObjektFactory factory = DObjPlugin.getDefault()
				.getObjektFactory();
		final List<? extends SystemObjekt> systemObjekte = factory
				.bestimmeModellobjekte(DarstellungsObjektTyp.PID);
		for (final SystemObjekt so : systemObjekte) {
			try {
				final EditorDoTypReferenz doTyp = DoeditorFactory.eINSTANCE
						.createEditorDoTypReferenz();
				doTyp.setSystemObjekt(so);
				typen.add(doTyp);
			} catch (final IllegalStateException exception) {
				Debug.getLogger().warning(
						"Der DOT: " + so.getName()
								+ " konnte nicht geladen werden!: "
								+ exception.getLocalizedMessage());
			}
		}

		return typen;
	}

	/**
	 * Gibt alle Darstellungsobjekttypen zurck, die zur fr einen bestimmten
	 * Systemobjekttyp Verfgung stehen. Es werden die Darstellungsobjekttypen
	 * zurckgegegeben, die exakt den angegebenen Systemobjekttyp besitzen.
	 * 
	 * @param systemObjectType
	 *            der Systemobjekttyp fr den die passenden Darstellungsobjekten
	 *            gesucht sind.
	 * @return alle bekannten Darstellungsobjekttypen zum bergebenen
	 *         Systemobjekttyp.
	 */
	public static Collection<DoTyp> getDoTypenWithExactMatch(
			final SystemObjectType systemObjectType) {
		if (systemObjectType == null) {
			throw new IllegalArgumentException("Argument darf nicht null sein.");
		}

		// Programmierte DO-Typen
		final Collection<DoTyp> filteredTypen = new ArrayList<DoTyp>();
		filteredTypen.addAll(DarstellungExtensionPoint
				.getDoTypenWithExactMatch(systemObjectType));

		// Editor-DO-Typen
		final ObjektFactory factory = DObjPlugin.getDefault()
				.getObjektFactory();
		final List<? extends SystemObjekt> systemObjekte = factory
				.bestimmeModellobjekte(DarstellungsObjektTyp.PID);
		for (final SystemObjekt so : systemObjekte) {
			final EditorDoTypReferenz doTyp = DoeditorFactory.eINSTANCE
					.createEditorDoTypReferenz();
			doTyp.setSystemObjekt(so);
			if (systemObjectType.equals(doTyp.getSystemObjectType())) {
				filteredTypen.add(doTyp);
			}
		}

		return filteredTypen;
	}

	/**
	 * Gibt alle Darstellungsobjekttypen zurck, die zur fr einen bestimmten
	 * Systemobjekttyp Verfgung stehen. Es werden die Darstellungsobjekttypen
	 * zurckgegegeben, die angegebenen Systemobjekttyp oder einem seiner
	 * Supertypen besitzen.
	 * 
	 * @param systemObjectType
	 *            der Systemobjekttyp fr den die passenden Darstellungsobjekten
	 *            gesucht sind.
	 * @return alle bekannten Darstellungsobjekttypen zum bergebenen
	 *         Systemobjekttyp.
	 */
	public static Collection<DoTyp> getDoTypen(
			final SystemObjectType systemObjectType) {
		if (systemObjectType == null) {
			throw new IllegalArgumentException("Argument darf nicht null sein.");
		}

		// Programmierte DO-Typen
		final Collection<DoTyp> filteredTypen = new ArrayList<DoTyp>();
		filteredTypen.addAll(DarstellungExtensionPoint
				.getDoTypen(systemObjectType));

		// Editor-DO-Typen
		final ObjektFactory factory = DObjPlugin.getDefault()
				.getObjektFactory();
		final List<? extends SystemObjekt> systemObjekte = factory
				.bestimmeModellobjekte(DarstellungsObjektTyp.PID);
		for (final SystemObjekt so : systemObjekte) {
			final EditorDoTypReferenz doTyp = DoeditorFactory.eINSTANCE
					.createEditorDoTypReferenz();
			doTyp.setSystemObjekt(so);
			if (systemObjectType.equals(doTyp.getSystemObjectType())
					|| systemObjectType.getSuperTypes().contains(
							doTyp.getSystemObjectType())) {
				filteredTypen.add(doTyp);
			}
		}

		return filteredTypen;
	}

	/**
	 * Gibt einen programmierten Darstellungsobjekttyp anhand seiner Extension
	 * ID zurck.
	 * 
	 * @param extensionId
	 *            die ID der Extension mit der der Darstellungsobjekttyp
	 *            defniert wurde.
	 * @return der Darstellungsobjekttyp oder <code>null</code>, wenn zu der ID
	 *         keine Extension fr einen Darstellungsobjekt existiert.
	 */
	public static ConfiguratedDoTyp getConfiguratedDoTyp(
			final String extensionId) {
		return DarstellungExtensionPoint.getDoTyp(extensionId);
	}

	/**
	 * Gibt einen Editor-Darstellungsobjekttyp anhand seiner PID zurck.
	 * 
	 * @param pid
	 *            die PID des Systemobjekts im Datenverteiler mit der der
	 *            Darstellungsobjekttyp defniert wurde.
	 * @return der Darstellungsobjekttyp oder <code>null</code>, wenn zu der PID
	 *         keine Darstellungsobjekt in der Konfiguration existiert.
	 */
	public static EditorDoTypReferenz getEditorDoTyp(final String pid) {
		final EditorDoTypReferenz doTyp = DoeditorFactory.eINSTANCE
				.createEditorDoTypReferenz();
		final SystemObjekt so = DObjPlugin.getDefault().getObjektFactory()
				.getModellobjekt(pid);
		doTyp.setSystemObjekt(so);
		return doTyp;
	}

}
